<?php

class Chi2FeatureSubset
{
    protected $p = 0.06;

    public function select($n, $labels, $features)
    {
        $selected = new SplMinHeap();
        $allowed = intval(ceil($this->p * $n));

        // Start with 1, since 0 is dedicated to the constant intercept term
        for ($j = 1; $j < $n; $j++) {
            $maxChi2 = 0.0;
            foreach ($labels as $label) {
                $stats = $features->varStats($j, $label);
                list($t_l, $t_nl, $nt_l, $nt_nl) = $stats;
                $num = ($t_l * $nt_nl) - ($t_nl * $nt_l);
                $den = ($t_l + $t_nl) * ($nt_l + $nt_nl);
                $chi2 = ($num * $num) / $den;
                if ($chi2 > $maxChi2) {
                    $maxChi2 = $chi2;
                }
            }

            if ($allowed > 0) {
                $selected->insert(array($maxChi2, $j));
                $allowed -= 1;
            } else {
                list($otherChi2, $_) = $selected->top();
                if ($maxChi2 > $otherChi2) {
                    $selected->extract();
                    $selected->insert(array($maxChi2, $j));
                }
            }
        }

        $keepFeatures = array(0 => 0);
        $i = 1;
        while (!$selected->isEmpty()) {
            list($chi2, $j) = $selected->extract();
            $keepFeatures[$j] = $i++;
        }

        return $keepFeatures;
    }
}
