<?php

require_once 'Features.php';

class TestFeatures extends PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $this->input = array(
            array(array("a", "b", "b"), "doc1"),
            array(array("c", "a", "d"), "doc2"),
            array(array("a", "b", "c"), "doc3")
        );
    }

    public function testBinaryFeaturesRead()
    {
        $binFeatures = new BinaryFeatures();
        foreach ($this->input as $doc) {
            list($terms, $label) = $doc;
            $binFeatures->newExample($label);
            foreach ($terms as $term) {
                $binFeatures->addTerm($term);
            }
            $binFeatures->endExample();
        }
        list($X, $y, $labels) = $binFeatures->endExamples();

        $this->assertSame($binFeatures->nameForVar(0), 'a');
        $this->assertSame($binFeatures->nameForVar(1), 'b');
        $this->assertSame($binFeatures->nameForVar(2), 'c');
        $this->assertSame($binFeatures->nameForVar(3), 'd');

        $this->assertSame($binFeatures->nameForLabel(0), 'doc1');
        $this->assertSame($binFeatures->nameForLabel(1), 'doc2');
        $this->assertSame($binFeatures->nameForLabel(2), 'doc3');

        $this->assertSame($y, array(0, 1, 2));
        $this->assertSame($X->rows(), 3);
        $this->assertSame($X->columns(), 4);
        $this->assertSame($X->nonzero(), 8);

        return array($X, $y, $labels);
    }

    /** @depends testBinaryFeaturesRead */
    public function testSparseMatrixIteration($data)
    {
        list($X, $y, $labels) = $data;
        $flat = iterator_to_array(new IteratorIterator($X));
        $this->assertSame(array_values($flat), array(
            array(0, 0, 1), // 1 'a'
            array(0, 1, 1), // 2 'b'

            array(1, 0, 1), // 1 'a'
            array(1, 2, 1), // 1 'c'
            array(1, 3, 1), // 1 'd'

            array(2, 0, 1), // 1 'a'
            array(2, 1, 1), // 1 'b'
            array(2, 2, 1)  // 1 'c'
        ));
    }
}
