<?php

require_once 'Features.php';
require_once 'LogisticRegression.php';

class TestInvertedData extends PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $data = array(
            array(1, 0, 0, 1, 1, 0),
            array(0, 0, 1, 0, 1, 0),
            array(1, 0, 0, 0, 1, 1),
            array(0, 0, 0, 1, 0, 0),
            array(0, 0, 0, 0, 1, 0),
            array(1, 0, 0, 0, 0, 1),
        );

        $X = new SparseMatrix(6, 6);
        foreach ($data as $i => $entries) {
            $row = array();
            foreach ($entries as $j => $present)
                if ($present)
                    $row[$j] = $present;
            $X->setRow($i, $row);
        }
        $this->invData = new InvertedData($X);
    }

    public function testSummaryStatistics()
    {
        $columns = $this->invData->columns();
        $this->assertEquals($columns, 6);

        $rows = $this->invData->rows();
        $this->assertEquals($rows, 6);
    }

    public function testIterateColumn()
    {
        // A column with 1+ rows non-zero values
        $itr = $this->invData->iterateColumn(2);
        $arr = itrToArray($itr);
        $this->assertSame($arr, array('2,1,1'));

        // A column with no non-zero values
        $itr = $this->invData->iterateColumn(1);
        $arr = iterator_to_array($itr);
        $this->assertEmpty($arr);

        // The last column
        $itr = $this->invData->iterateColumn(5);
        $arr = itrToArray($itr);
        $this->assertSame($arr, array('5,2,1', '5,5,1'));
    }
}


///////////////////////////////////////////////////////////////////////////////
//                             UTILITY FUNCTIONS                             //
///////////////////////////////////////////////////////////////////////////////

function itrToArray($itr) {
    $arr = array();
    foreach ($itr as $entry) {
        list($j, $i, $x) = $entry;
        $arr[] = "{$j},{$i},{$x}";
    }
    return $arr;
}
