<?php

require_once 'Classifier.php';

class NaiveBayes extends Classifier
{
    protected $gamma = 1;
    protected $epsilon = 1;
    protected $Beta = array();

    /**
     * Train on a set of documents.
     */
    public function train($X, $y, $labels)
    {
        $this->handleBinaryLabels($labels);

        foreach ($labels as $label) {
            $targetY = array();
            foreach ($y as $i => $yi)
                $targetY[] = $yi == $label ? 1 : 0;

            $this->trainLabel($X, $targetY, $label);
        }
    }

    /**
     * Classify an instance and return the name of the best class.
     */
    public function classify($x, $threshold = 0.5)
    {
        $maxScore = -INF;
        $bestLabel = '';

        foreach (array_keys($this->Beta) as $label) {
            $Beta =& $this->Beta[$label];
            $l = 0.0;
            foreach ($x as $j => $xj) {
                $l += $Beta[$j] * $xj;
            }
            $score = 1.0 / (1.0 + exp(-$l));

            if ($score > $maxScore) {
                $maxScore = $score;
                $bestLabel = $label;
            }
        }

        if (count($this->Beta) == 1 && $maxScore <= $threshold) {
            $bestLabel = $this->negLabel;
            $maxScore = 1 - $maxScore;
        }

        return array($bestLabel, $maxScore);
    }


    ///////////////////////////////////////////////////////////////////////////
    //                         Private Interface                             //
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Train on a particular label.
     */
    protected function trainLabel($X, $y, $label)
    {
        $n = $X->columns();
        $p = array_fill(0, $n, 0);
        $a = array_fill(0, $n, 0);

        $this->Beta[$label] = array_fill(0, $n, 0.0);
        $Beta =& $this->Beta[$label];

        foreach ($X as $cell) {
            list($i, $j, $Xij) = $cell;
            fprintf(STDERR, "\ri = %4d, j = %4d", $i, $j);

            if ($y[$i] == 1)
                $p[$j] += 1;
            else
                $a[$j] += 1;
        }

        fprintf(STDERR, "\ncomputing beta from term frequencies\n");
        $Beta[0] = $this->logit($p[0], $a[0]);
        for ($j = 1; $j < $n; $j++) {
            $Beta[$j] = $this->logit($p[$j], $a[$j]) - $Beta[0];
            fprintf(STDERR, "\rj = %4d, beta = %0.4f", $j, $Beta[$j]);
        }
        fprintf(STDERR, "\n");
    }

    protected function logit($pos, $neg)
    {
        $odds = ($pos + $this->gamma) / ($neg + $this->epsilon);
        $p = $odds / (1.0 + $odds);
        return log($p / (1.0 - $p));
    }
}
