#!/usr/bin/env php
<?php

/**
 * A small script to filter the output of nbayes.php on syscall classification. 
 * Use the names of the files (section 2 files end in .2 and section 3 files 
 * end in .3) as an oracle for the correct class, and compare this with the 
 * predicted class. Print nothing for correct classifications, and prepend an 
 * 'X' for incorrect ones. At the end, print the accuracy.
 *
 * Example usage:
 *
 *    $ php nbayes.php --classify DIR --db DBFILE | php eval_accuracy.php
 */

function main()
{
    $total = 0;
    $correct = 0;
    while (($line = fgets(STDIN)) !== false) {
        sscanf($line, "%f %s %s", $score, $file, $label);
        $actual = preg_replace('/^.*\.(.*)$/', '$1', $file);
        if ($label == $actual)
            $correct += 1;
        else
            printf("%.3f %s\n", $score, $file);
        $total += 1;
    }

    printf("%d/%d (%.4f)\n", $correct, $total, $correct / $total);
}

main();
